@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2007, Benjamin Kaufmann
+// Copyright (c) 2006-2010, Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -26,85 +26,78 @@
 
 #include <clasp/solver.h>
 #include <clasp/util/indexed_priority_queue.h>
+#include <ctime>
 
 namespace Clasp { namespace SatElite {
 
-class Clause; // Special clause class optimized for use with the preprocessor
-
 //! SatElite preprocessor for clauses
 /*!
+ * The preprocessor implements subsumption, self-subsumption, variable elimination,
+ * and (optionally) blocked clause elimination.
  * \see 
  *   - Niklas En, Armin Biere: "Effective Preprocessing in SAT through Variable and Clause Elimination" 
+ *   - Matti Jrvisalo, Armin Biere, Marijn Heule: "Blocked Clause Elimination"
  *   - Parts of the SatElite preprocessor are adapted from MiniSAT 2.0 beta
  *     available under the MIT licence from http://minisat.se/MiniSat.html
  *   .
  */
 class SatElite : public Clasp::SatPreprocessor {
 public:
-	explicit SatElite(Solver* s = 0);
+	SatElite();
 	~SatElite();
-
-	//! adds a clause to the preprocessor
-	/*!
-	 * \pre clause is not a tautology (i.e. does not contain both l and ~l)
-	 * \pre clause is a set (i.e. does not contain l more than once)
-	 * \return true if clause was added. False if adding the clause makes the problem UNSAT
-	 */
-	bool addClause(const LitVec& clause);
-
-	//! Options used by the preprocessor
-	struct Options {
-		Options() : maxFrozen(1.0), maxTime(UINT32_MAX), maxIters(UINT32_MAX), maxOcc(UINT32_MAX), elimPure(true) {}
-		double  maxFrozen;/**< run only if #frozen/#vars <= maxFrozen           */
-		uint32  maxTime;  /**< maximal runtime, checked after each iteration    */
-		uint32  maxIters; /**< maximal number of iterations                     */
-		uint32  maxOcc;   /**< skip v, if #occ(v) > maxOcc && #occ(~v) > maxOcc */
-		bool    elimPure; /**< eliminate pure literals?                         */
-	} options;
-
-	bool preprocess();
-	void extendModel( Assignment& m );
-	void clearModel() { unconstr_.clear(); }
-	bool hasSymModel() const { return !unconstr_.empty(); }
-	void setEnumerate(bool b){ options.elimPure = !b; }
-	typedef PodVector<Clause*>::type  ClauseList;
+	Clasp::SatPreprocessor* clone();
+	struct Progress : public Event_t<Progress> {
+		enum EventOp { event_algorithm = '*', event_bce = 'B', event_var_elim = 'E', event_subsumption = 'S', };
+		Progress(SatElite* p, EventOp o, uint32 i, uint32 m) : Event_t<Progress>(Event::subsystem_prepare, Event::verbosity_high), self(p), cur(i), max(m) {
+			op = (uint32)o;
+		}
+		SatElite* self;
+		uint32    cur;
+		uint32    max;
+	};
+protected:
+	bool  initPreprocess(Options& opts);
+	void  reportProgress(Progress::EventOp, uint32 curr, uint32 max);
+	bool  doPreprocess();
+	void  doExtendModel(ValueVec& m, LitVec& open);
+	void  doCleanUp();
 private:
 	typedef PodVector<uint8>::type    TouchedList;
+	typedef bk_lib::left_right_sequence<Literal, Var,0> ClWList;
+	typedef ClWList::left_iterator                      ClIter;
+	typedef ClWList::right_iterator                     WIter;
+	typedef std::pair<ClIter, ClIter>                   ClRange;
 	SatElite(const SatElite&);
 	const SatElite& operator=(const SatElite&);
-	struct ElimData {
-		typedef Clause** ClauseArray; 
-		ElimData(Var v) : eliminated(0), var(v) {}
-		ClauseArray eliminated; // null-terminated array of clauses containing var
-		Var         var;        // eliminated var
-	};
-	typedef PodVector<ElimData>::type ElimList;
 	// For each var v
 	struct OccurList {
-		OccurList() : pos(0), dirty(0), neg(0), litMark(0) {}
-		LitVec  clauses;    // ids of clauses containing v or ~v  (var() == id, sign() == v or ~v)
-		VarVec  watches;    // ids of clauses watching v or ~v (literal 0 is the watched literal)
-		uint32  pos:31;     // number of *relevant* clauses containing v
-		uint32  dirty:1;    // does clauses contain removed clauses?
-		uint32  neg:30;     // number of *relevant* clauses containing v
-		uint32  litMark:2;  // 00: no literal of v marked, 01: v marked, 10: ~v marked
+		OccurList() : pos(0), bce(0), dirty(0), neg(0), litMark(0) {}
+		ClWList refs;      // left : ids of clauses containing v or ~v  (var() == id, sign() == v or ~v)
+		                   // right: ids of clauses watching v or ~v (literal 0 is the watched literal)
+		uint32  pos:30;    // number of *relevant* clauses containing v
+		uint32  bce:1;     // in BCE queue?
+		uint32  dirty:1;   // does clauses contain removed clauses?
+		uint32  neg:30;    // number of *relevant* clauses containing v
+		uint32  litMark:2; // 00: no literal of v marked, 01: v marked, 10: ~v marked
 		uint32  numOcc()          const { return pos + neg; }
 		uint32  cost()            const { return pos * neg; }
+		ClRange clauseRange()     const { return ClRange(const_cast<ClWList&>(refs).left_begin(), const_cast<ClWList&>(refs).left_end()); }
 		void    clear() {
 			this->~OccurList();
 			new (this) OccurList();
 		}
+		void    addWatch(uint32 clId)    { refs.push_right(clId); }
+		void    removeWatch(uint32 clId) { refs.erase_right(std::find(refs.right_begin(), refs.right_end(), clId)); }
 		void    add(uint32 id, bool sign){
 			pos += uint32(!sign);
 			neg += uint32(sign);
-			clauses.push_back(Literal(id, sign));
+			refs.push_left(Literal(id, sign));
 		}
 		void    remove(uint32 id, bool sign, bool updateClauseList) {
 			pos -= uint32(!sign);
 			neg -= uint32(sign);
 			if (updateClauseList) { 
-				LitVec::iterator it = std::find(clauses.begin(), clauses.end(), Literal(id, sign));
-				if (it != clauses.end()) { clauses.erase(it); }
+				refs.erase_left(std::find(refs.left_begin(), refs.left_end(), Literal(id, sign)));
 			}
 			else { dirty = 1; }
 		}
@@ -123,45 +116,52 @@ private:
 	typedef bk_lib::indexed_priority_queue<LessOccCost> ElimHeap;
 	Clause*         peekSubQueue() const {
 		assert(qFront_ < queue_.size());
-		return clauses_[ queue_[qFront_] ];
+		return (Clause*)clause( queue_[qFront_] );
 	}
 	inline Clause*  popSubQueue();
 	inline void     addToSubQueue(uint32 clauseId);
 	void            updateHeap(Var v) {
-		assert(solver_);
-		if (!solver_->frozen(v) && !solver_->eliminated(v)) {
+		assert(ctx_);
+		if (!ctx_->frozen(v) && !ctx_->eliminated(v)) {
 			elimHeap_.update(v);
+			if (occurs_[v].bce == 0 && occurs_[0].bce != 0) {
+				occurs_[0].addWatch(v);
+				occurs_[v].bce = 1;
+			}
 		}
 	}
 	inline uint32   findUnmarkedLit(const Clause& c, uint32 x) const;
 	void    attach(uint32 cId, bool initialClause);
-	void    detach(uint32 cId, bool destroy);
+	void    detach(uint32 cId);
+	void    bceVeRemove(uint32 cId, bool freeId, Var v, bool blocked);
 	bool    propagateFacts();
 	bool    backwardSubsume();
 	Literal subsumes(const Clause& c, const Clause& other, Literal res) const;
 	bool    strengthenClause(uint32 clauseId, Literal p);
 	bool    subsumed(LitVec& cl);
-	bool    eliminateVar(Var v);
-	bool    trivialResolvent(const Clause& c1, const Clause& c2, Var v) const;
-	bool    addResolvent(const Clause& c1, const Clause& c2, Var v);
-
-	uint32  freeClauseId();
-	void    cleanUp();
-
-	typedef std::pair<uint32, uint32> ElimPos;
+	bool    eliminateVars();
+	bool    bce();
+	bool    bceVe(Var v, uint32 maxCnt);
+	ClRange splitOcc(Var v, bool mark);
+	bool    trivialResolvent(const Clause& c2, Var v) const;
+	void    markAll(const Literal* lits, uint32 size) const;
+	void    unmarkAll(const Literal* lits, uint32 size) const;
+	bool    addResolvent(uint32 newId, const Clause& c1, const Clause& c2);
+	bool    cutoff(Var v) const {
+		return opts_->occLimit(occurs_[v].pos, occurs_[v].neg)
+		  ||   (occurs_[v].cost() == 0 && opts_->mode == Options::prepro_preserve_models);
+	}
+	bool    timeout() const { return time(0) > timeout_; }
+	const Options*opts_;      // active options
 	OccurList*    occurs_;    // occur list for each variable
-	ElimList*     elimList_;  // stores eliminated vars and the clauses in which they occur
-	LitVec        unconstr_;  // eliminated vars that are unconstraint w.r.t the current model
 	ElimHeap      elimHeap_;  // candidates for variable elimination; ordered by increasing occurrence-cost
-	ClauseList    clauses_;   // all clauses
-	ClauseList  posT_, negT_; // temporary clause lists used in eliminateVar
+	VarVec      posT_, negT_; // temporary clause lists used in eliminateVar
 	ClauseList    resCands_;  // pairs of clauses to be resolved
 	LitVec        resolvent_; // temporary, used in addResolvent
 	VarVec        queue_;     // indices of clauses waiting for subsumption-check
-	ElimPos       elimPos_;   // position of eliminated clause whose id is to be reused next
 	uint32        qFront_;    // front of queue_, i.e. [queue_.begin()+qFront_, queue.end()) is the subsumption queue
 	uint32        facts_;     // [facts_, solver.trail.size()): new top-level facts
-	uint32        clRemoved_; // number of clauses removed since last compaction
+	std::time_t   timeout_;   // stop once time > timeout_
 };
 }}
 #endif
